package Program;

################################################################################
# PAD Archiv Demo
# Copyright (c) 2000-2002 by Oliver Grahl Software Design
#
# http://www.o-grahl.de/deupad/
#
# Freeware, Verwendung auf eigene Gefahr.
# Verffentlichung oder Weitergabe nur mit Zustimmung des Autors.
################################################################################
#
# Program.pm
#
# Zweck:
#  Datenklasse fuer Programme
# Funktionsweise:
#  - Konstruktion mit $Program = Program->new();
#  - Attribute: padurl       URL der PAD Datei
#               name         Programmname
#               version      Programmversion
#               description  Programmbeschreibung
#               category     Programmkategorie
#               errorcount   Fehlerzaehler beim Spidern
#               lasterror    Letzter Fehler beim Spidern
#  - Attribute lesen mit z.B. $var = $Program->name();
#  - Attribute schreiben mit z.B. $Program->name("Neuer Wert");
#  - $Program->csv() gibt die Felder ;-separiert zurueck
#    (fuers Schreiben in CSV-Datenbank)
#  - $Program->csv($line) parst die Felder aus einer ;-separierten Zeile
#    (fuers Lesen aus CSV-Datenbank)
#  Hinweis: Wegen des einfachen ;-separierten Datenbankformats ohne Quoting
#           werden alle Strickpunkte aus den Feldern geloescht.
#
################################################################################

use strict;


##############################################
## Konstruktion                             ##
##############################################
sub new {
  my $self = {};

  $self->{PADURL} = undef;
  $self->{NAME} = undef;
  $self->{VERSION} = undef;
  $self->{DESCRIPTION} = undef;
  $self->{CATEGORY} = undef;
  $self->{ERRORCOUNT} = 0;
  $self->{LASTERROR} = "";

  bless($self);
  return $self;
}


##############################################
## Attribute                                ##
##############################################
sub padurl {
  my $self = shift;
  if (@_) { $self->{PADURL} = shift }
  return $self->{PADURL};
}
sub name {
  my $self = shift;
  if (@_) { $self->{NAME} = shift; $self->{NAME} =~ s/;//g; }
  return $self->{NAME};
}
sub version {
  my $self = shift;
  if (@_) { $self->{VERSION} = shift; $self->{VERSION} =~ s/;//g; }
  return $self->{VERSION};
}
sub description {
  my $self = shift;
  if (@_) { $self->{DESCRIPTION} = shift; $self->{DESCRIPTION} =~ s/;//g; }
  return $self->{DESCRIPTION};
}
sub category {
  my $self = shift;
  if (@_) { $self->{CATEGORY} = shift; $self->{CATEGORY} =~ s/;//g; }
  return $self->{CATEGORY};
}
sub errorcount {
  my $self = shift;
  if (@_) { $self->{ERRORCOUNT} = shift }
  return $self->{ERRORCOUNT};
}
sub lasterror {
  my $self = shift;
  if (@_) { $self->{LASTERROR} = shift; $self->{LASTERROR} =~ s/;//g; }
  return $self->{LASTERROR};
}


##############################################
## Methoden                                 ##
##############################################
sub csv {
  my $self = shift;
  if (@_) {
    my @line = split(/;/, shift);
    $self->{PADURL} = $line[0];
    $self->{ERRORCOUNT} = $line[1];
    $self->{LASTERROR} = $line[2];
    $self->{NAME} = $line[3];
    $self->{VERSION} = $line[4];
    $self->{DESCRIPTION} = $line[5];
    $self->{CATEGORY} = $line[6];
  }
  return $self->{PADURL} . ";" .
         $self->{ERRORCOUNT} . ";" .
         $self->{LASTERROR} . ";" .
         $self->{NAME} . ";" .
         $self->{VERSION} . ";" .
         $self->{DESCRIPTION} . ";" .
         $self->{CATEGORY} . ";";
}


1;
