#!/bin/perl

################################################################################
# PAD Archiv Demo
# Copyright (c) 2000-2004 by GRAHL software design
#
# http://www.DeuPAD.org
#
# Freeware, Verwendung auf eigene Gefahr.
# Verffentlichung oder Weitergabe nur mit Zustimmung des Autors.
################################################################################
#
# anmelden.pl
#
# Zweck:
#  Eintragen eines Programms in die Datenbank durch Anmelden einer PAD Url
# Funktionsweise:
#  - Gibt beim ersten Aufruf ein Formular zur Eingabe der PAD Url und
#    Auswahl einer Programmkategorie aus.
#  - Nimmt nach Absenden eines ausgefuellten Formulars die Eintragung vor:
#  - Lese die PAD Datei von der angegebenen URL ein.
#  - Kann die Datei nicht gelesen werden, Fehlermeldung ausgeben
#  - Pruefe den Inhalt der PAD Datei
#  - Lese die Datenbank ein und pruefe, ob die URL schon einmal eingetragen
#    wurde
#  - Wenn URL noch nicht in Datenbank: Lege ein neues Programm fuer die
#    Datenbank an und lese die benoetigten Informationen aus der PAD Datei aus.
#  - Wenn URL schon in Datenbank, aktualisiere die Daten aus der PAD Datei.
#  - Schreibe die Datenbank mit dem neuen Datensatz.
#
################################################################################

use lib $ENV{'DOCUMENT_ROOT'} . '/cgi-bin/padarc';  # Siehe readme.txt !!

use Program;
use Database;
use PadFile;


print "Content-type: text/html\n\n";

# Lese Formularfelder ein und entnehme die Felder PADUrl und Category
&getFields();

my $padurl = $FIELDS{'PADUrl'};
my $category = $FIELDS{'Category'};


# Gebe Formular aus, falls keine PADUrl angegeben
if ( !$padurl ) {
  # HTML Formular ausgeben
  print qq!
  <html>

  <head>
  <title>PAD Archiv Demo - Anmelden</title>
  </head>

  <body><h1>PAD Archiv Demo</h1>
  <p><a href="anzeigen.pl">Alle Programme</a> - <b>Programm anmelden</b> - <a href="spidern.pl">Spidern</a></p>

  <form method="POST" action="anmelden.pl">
    <p>Bitte geben Sie die PAD Url ein <br>
    (z.B. http://www.hersteller.com/produkt/pad_file.xml):</p>
    <p><input type="text" name="PADUrl" size="60"></p>
    <p>Bitte whlen Sie die Kategorie fr das Produkt:</p>
    <p><select size="1" name="Category">
      <option>Tools</option>
      <option>Office</option>
      <option>Spiele</option>
    </select></p>
    <p>Klicken Sie auf &quot;Eintragen&quot; und warten Sie auf die Besttigung:</p>
    <p><input type="submit" value="Eintragen\!" name="Eintragen"></p>
  </form>

  </body>

  </html>
  !;

  exit;
}


# HTML Kopf ausgeben
print qq!
<html>

<head>
<title>PAD Archiv Demo - Anmelden</title>
</head>

<body><h1>PAD Archiv Demo</h1>
<p><a href="anzeigen.pl">Alle Programme</a> - <a href="anmelden.pl">Programm anmelden</a> - <a href="spidern.pl">Spidern</a></p>
!;


# Eine PADUrl wurde angegeben:
# Erzeuge ein PadFile Objekt und lade die PAD Datei
my $PadFile = PadFile->new();
$PadFile->padurl($padurl);
if ( $PadFile->readContent() ) {
  # PAD Datei konnte gelesen werden
  print "PAD Datei wurde gefunden.<br>";

  # Verifiziere Feld Application_XML_File_URL
  # Das Feld muss die URL enthalten, so wie sie hier angemeldet wird.
  # Ist das nicht der Fall, schlgt das spaetere Spidern fehl.
  $field_pad_url = $PadFile->getValue("XML_DIZ_INFO", "Web_Info", "Application_URLs", "Application_XML_File_URL");
  if ( $field_pad_url ne $padurl ) {
    # Das Feld Application_XML_File_URL ist nicht mit der angemeldeten URL identisch
    # -> Keine Eintragung zulassen, Fehlermeldung ausgeben
    print "Bei dem Versuch, die PAD Datei $padurl anzumelden ist ein Fehler aufgetreten. Korrigieren Sie den Fehler und versuchen Sie es erneut.<br><br>";
    print "Fehler: Das Feld XML_DIZ_INFO/Web_Info/Application_URLs/Application_XML_File_URL in der PAD Datei enthlt die Adresse $field_pad_url, die nicht mit der anzumeldenden Adresse bereinstimmt.";
  }
  else {
    # Lese Database ein und sehe nach, ob die Url schon einmal eingetragen wurde
    my $NewProgram = undef;
    my $IsNewProgram = 1;
    my $Database = Database->new();
    $Database->readData();
    foreach my $Program (@{$Database->programs()}) {
     if ( $Program->padurl() eq $padurl ) {
       # Die URL wurde in der Datenbank gefunden, das Programm ist also
       # schon einmal in die Datenbank eingetragen worden.
       $IsNewProgram = 0;
       $NewProgram = $Program;
       print "Programm existiert bereits in Datenbank, Daten werden aktualisiert...<br>";
       last;
     }
    }

    # Falls Programm noch nicht existierte, neues Program Objekt erzeugen
    if ( $IsNewProgram ) {
      $NewProgram = Program->new();
    }

    # Program Objekt mit den Daten fuellen
    $NewProgram->padurl($padurl);
    $NewProgram->category($category);
    $NewProgram->name($PadFile->getValue("XML_DIZ_INFO", "Program_Info", "Program_Name"));
    $NewProgram->version($PadFile->getValue("XML_DIZ_INFO", "Program_Info", "Program_Version"));
    my $descr = $PadFile->getValue("XML_DIZ_INFO", "Program_Descriptions", "German", "Char_Desc_45");
    if ( $descr eq "" ) { $descr = $PadFile->getValue("XML_DIZ_INFO", "Program_Descriptions", "German", "Char_Desc_80"); };
    if ( $descr eq "" ) { $descr = $PadFile->getValue("XML_DIZ_INFO", "Program_Descriptions", "English", "Char_Desc_45"); };
    if ( $descr eq "" ) { $descr = $PadFile->getValue("XML_DIZ_INFO", "Program_Descriptions", "English", "Char_Desc_80"); };
    $NewProgram->description($descr);

    # Fuege das neue Program Objekt der Datenbank hinzu
    if ( $IsNewProgram ) {
      push @{$Database->programs()}, $NewProgram;
    }
    # Schreibe die Datenbank
    $Database->writeData();

    print "Das Programm " . $NewProgram->name() . " wurde erfolgreich in die Datenbank eingetragen.";
  }
}
else {
  # PAD Datei konnte NICHT gelesen werden, Fehlermeldung ausgeben
  print "Bei dem Versuch, die PAD Datei $padurl anzumelden ist ein Fehler aufgetreten. Korrigieren Sie den Fehler und versuchen Sie es erneut.<br><br>";
  print "Fehler: " . $PadFile->error();
}


# HTML Fuss ausgeben
print qq!
</body>

</html>
!;


##########################################################
# Formulardaten in Hash @FIELDS einlesen                 #
##########################################################
sub getFields
{
  if ($ENV{'REQUEST_METHOD'} eq 'GET') {
    @pairs = split(/&/, $ENV{'QUERY_STRING'});
  }
  elsif ($ENV{'REQUEST_METHOD'} eq 'POST') {
    read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
    @pairs = split(/&/, $buffer);
  }
  else {
    die "Request Method not allowed";
  }

  foreach $pair (@pairs) {
    ($name, $value) = split(/=/, $pair);

    $name =~ tr/+/ /;
    $name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;

    $value =~ tr/+/ /;
    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;

    $value =~ s/<!--(.|\n)*-->//g;

    $FIELDS{$name} = $value;
  }
}
